/*
  $Id: example.cc 5788 2010-10-22 10:09:57Z abehm $

  Copyright (C) 2010 by The Regents of the University of California
	
  Redistribution of this file is permitted under
  the terms of the BSD License.
    
  Date: 09/06/2008
  Author: Alexander Behm <abehm (at) ics.uci.edu>
*/

#include "wrappers.h"
using namespace std;

void readStrPair(vector<pair<string,uint> >& data, const string& filename, unsigned count, unsigned maxLineLen)
{
	ifstream fin(filename.c_str());
	if (! fin)
	{
		cerr << "Cannot open string file " << filename << endl;
	}
	while (fin)
	{
		pair<string, uint> tmp;
		float x,y;
		fin >> tmp.second >> x>> y>> tmp.first;
		if (! fin.good()) continue;
		data.push_back(tmp);
	}
	fin.close();
}

// create a dummy dictionary
extern void readString(vector<string>& data, const string& filenameData, unsigned count, unsigned maxLineLen);
std::vector<string> dictionary;
std::vector<pair<string, uint> > strPairColl;
void initDictionary();

// EXAMPLES
void wrapperSimpleExampleEd();
void wrapperSimpleShortExampleEd();
void wrapperSimpleExampleEdNorm();
void wrapperSimpleIndexMaintenance();
void wrapperDiscardListsExample();
void wrapperOnDiskSimpleExample();
void customerizeMerger(); 

int main() {
  
  //initDictionary();
  readStrPair(strPairColl, "/home/tang/netsasFeb15/data/na.str",1983155,100);
  cout<<strPairColl.size()<<endl;
  
/*
  wrapperSimpleExampleEd();
  wrapperSimpleShortExampleEd();
  wrapperSimpleExampleEdNorm();
  wrapperSimpleIndexMaintenance();
  wrapperDiscardListsExample();
*/  
  //wrapperOnDiskSimpleExample();
  customerizeMerger() ;
  
  return 0;
}

/*
void initDictionary() {
  // params: target vector, filename, number strings to read, max line length
  readString(dictionary, "../data/female_names.txt", 4000, 20);
}

// USAGE OF WRAPPERS FOR UNCOMPRESSED INDEXES
// List of Wrappers:
// WrapperSimpleEd
// WrapperSimpleEdNorm
// WrapperSimpleJacc
// WrapperSimpleCos
// WrapperSimpleDice
// see typedefs in wrapperuncompressed.h
// ALSO POSSIBLE TO SPECIFY SIMMETRIC AS TEMPLATE PARAMETER: WrapperSimple<SimilarityMetric>
void wrapperSimpleExampleEd() {
  cout << "----- EXAMPLE: WrapperSimpleEd -----" << endl;
  
  GramGenFixedLen gramGen(2);
  
  StringContainerVector strContainer(true);
  strContainer.initStatsCollector(&gramGen);
  strContainer.fillContainer("../data/female_names.txt", 4000);
  
  // create wrapper and build index
  // params: stringcontainer, gramgenerator, use partitioning filter?
  WrapperSimpleEd wrapper(&strContainer, &gramGen, true);
  wrapper.buildIndex();
  
  // perform search
  float editDistance = 1.0f;
  string queryString = "kathrin";
  vector<unsigned> resultStringIDs;
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  // save index
  wrapper.saveIndex("wrapperIndex.ix");

  // load index
  wrapper.loadIndex("wrapperIndex.ix");

  // perform search on loaded index
  resultStringIDs.clear();
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  cout << "-----------------------------------------" << endl << endl;  
}

// USAGE OF WRAPPERS FOR UNCOMPRESSED INDEXES
// List of Wrappers:
// WrapperSimpleEd
// WrapperSimpleEdNorm
// WrapperSimpleJacc
// WrapperSimpleCos
// WrapperSimpleDice
// see typedefs in wrapperuncompressed.h
// ALSO POSSIBLE TO SPECIFY SIMMETRIC AS TEMPLATE PARAMETER: WrapperSimple<SimilarityMetric>
void wrapperSimpleShortExampleEd() {
  cout << "----- EXAMPLE: WrapperShortSimpleEd -----" << endl;
  
  GramGenFixedLen gramGen(2);
  
  StringContainerVector strContainer(true);
  strContainer.initStatsCollector(&gramGen);
  strContainer.fillContainer("../data/female_names.txt", 4000);
  
  // create wrapper and build index
  // params: stringcontainer, gramgenerator, use partitioning filter?
  WrapperShortSimpleEd wrapper(&strContainer, &gramGen, true);
  wrapper.buildIndex();
  
  // perform search
  float editDistance = 1.0f;
  string queryString = "kathrin";
  vector<unsigned> resultStringIDs;
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  // save index
  wrapper.saveIndex("wrapperIndex.ix");

  // load index
  wrapper.loadIndex("wrapperIndex.ix");

  // perform search on loaded index
  resultStringIDs.clear();
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  cout << "-----------------------------------------" << endl << endl;  
}

// USAGE OF WRAPPERS FOR UNCOMPRESSED INDEXES
// List of Wrappers:
// WrapperSimpleEd
// WrapperSimpleEdNorm
// WrapperSimpleJacc
// WrapperSimpleCos
// WrapperSimpleDice
// see typedefs in wrapperuncompressed.h
// ALSO POSSIBLE TO SPECIFY SIMMETRIC AS TEMPLATE PARAMETER: WrapperSimple<SimilarityMetric>
void wrapperSimpleExampleEdNorm() {
  cout << "----- EXAMPLE: WrapperSimpleEdNorm -----" << endl;
  
  GramGenFixedLen gramGen(2);
  
  StringContainerVector strContainer(true);
  strContainer.initStatsCollector(&gramGen);
  strContainer.fillContainer(dictionary.begin(), dictionary.end());

  // create wrapper and build index
  // params: stringcontainer, gramgenerator, use partitioning filter?
  WrapperSimpleEdNorm wrapper(&strContainer, &gramGen, true);
  wrapper.buildIndex();
  
  // perform search
  float normEditDistance = 0.9f;
  string queryString = "kathrin";
  vector<unsigned> resultStringIDs;
  wrapper.search(queryString, normEditDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  // save index
  wrapper.saveIndex("wrapperIndex.ix");

  // load index
  wrapper.loadIndex("wrapperIndex.ix");
  
  // perform search on loaded index
  resultStringIDs.clear();
  wrapper.search(queryString,  normEditDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  cout << "-----------------------------------------" << endl << endl;  
}

// USAGE OF WRAPPERS FOR UNCOMPRESSED INDEXES
// List of Wrappers:
// WrapperSimpleEd
// WrapperSimpleEdNorm
// WrapperSimpleJacc
// WrapperSimpleCos
// WrapperSimpleDice
// see typedefs in wrapperuncompressed.h
// ALSO POSSIBLE TO SPECIFY SIMMETRIC AS TEMPLATE PARAMETER: WrapperSimple<SimilarityMetric>
void wrapperSimpleIndexMaintenance() {
  cout << "----- EXAMPLE: WrapperSimpleEd Index Maintenance -----" << endl;
  
  GramGenFixedLen gramGen(2);
  
  StringContainerVector strContainer(true);
  strContainer.initStatsCollector(&gramGen);
  strContainer.fillContainer(dictionary.begin(), dictionary.end());

  // create wrapper and build index
  // params: stringcontainer, gramgenerator, use partitioning filter?
  WrapperSimpleEd wrapper(&strContainer, &gramGen, true);
  wrapper.buildIndex();
  
  // perform search
  float editDistance = 2.0f;
  string queryString = "kathrin";
  vector<unsigned> resultStringIDs;
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << resultStringIDs[i] << " " << tmp << endl;
  }
  
  // insert a couple of strings
  wrapper.insertString("kathryn");
  wrapper.insertString("kathein");
  wrapper.insertString("cathrin");
  wrapper.insertString("cathryn");
  
  resultStringIDs.clear();
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS AFTER INSERTIONS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << resultStringIDs[i] << " " << tmp << endl;
  }

  // delete first 1000 strings from index and stringcontainer
  for(unsigned i = 0; i < 1000; i++)
    wrapper.deleteString(i);
  
  resultStringIDs.clear();
  wrapper.search(queryString, editDistance, resultStringIDs);  
  cout << "SIMILAR STRINGS AFTER DELETIONS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << resultStringIDs[i] << " " << tmp << endl;
  }
  
  // integrate the deletions into the inverted lists
  // WARNING: this operation takes time and should be done periodically but infrequently
  wrapper.integrateUpdates();
  
  resultStringIDs.clear();
  wrapper.search(queryString, editDistance, resultStringIDs);  
  cout << "SIMILAR STRINGS AFTER INTEGRATION: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << resultStringIDs[i] << " " << tmp << endl;
  }


  cout << "-----------------------------------------" << endl << endl;  
}

// USAGE OF WRAPPERS FOR COMPRESSED INDEXES USING GLOBAL HOLES COMPRESSION
// List of Wrappers:

// Using Long-List-First hole selection (workload independent)
// WrapperDiscardListsLLFEd;
// WrapperDiscardListsLLFEdNorm;
// WrapperDiscardListsLLFJacc;
// WrapperDiscardListsLLFCos;
// WrapperDiscardListsLLFDice;

// Using Long-List-First hole selection (workload independent)
// WrapperDiscardListsSLFEd;
// WrapperDiscardListsSLFEdNorm;
// WrapperDiscardListsSLFJacc;
// WrapperDiscardListsSLFCos;
// WrapperDiscardListsSLFDice;

// Using Random-List hole selection (workload independent)
// WrapperDiscardListsRandomEd;
// WrapperDiscardListsRandomEdNorm;
// WrapperDiscardListsRandomJacc;
// WrapperDiscardListsRandomCos;
// WrapperDiscardListsRandomDice;

// Using Panic-Cost-Plus hole selection (workload dependent)
// WrapperDiscardListsPanicCostEd;
// WrapperDiscardListsPanicCostEdNorm;
// WrapperDiscardListsPanicCostJacc;
// WrapperDiscardListsPanicCostCos;
// WrapperDiscardListsPanicCostDice;

// Using Time-Cost-Plus hole selection (workload dependent)
// WrapperDiscardListsTimeCostEd;
// WrapperDiscardListsTimeCostEdNorm;
// WrapperDiscardListsTimeCostJacc;
// WrapperDiscardListsTimeCostCos;
// WrapperDiscardListsTimeCostDice;

// see typedefs in wrapperholesglobal.h
// ALSO POSSIBLE TO SPECIFY SIMMETRIC AS TEMPLATE PARAMETER: WrapperDiscardListsLLF<SimilarityMetric>
void wrapperDiscardListsExample() {
  cout << "----- EXAMPLE: WrapperDiscardLists ----" << endl;

  StringContainerVector strContainer(true);
  strContainer.fillContainer(dictionary.begin(), dictionary.end());
  
  // create wrapper and build index using 0.5 as reduction ratio
  WrapperDiscardListsLLFEd wrapper(&strContainer, 2, true, 0.5);
  wrapper.buildIndex();

  // perform search
  float editDistance = 1.0f;
  string queryString = "kathrin";
  vector<unsigned> resultStringIDs;
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  // save index
  wrapper.saveIndex("wrapperIndex.ix");

  // load index
  wrapper.loadIndex("wrapperIndex.ix");

  // perform search on loaded index
  resultStringIDs.clear();
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }

  cout << "-----------------------------------------" << endl << endl;  
}


// USAGE OF WRAPPERS FOR DISK-BASED INDEXES
// List of Wrappers:

// wrappers using simple merging algorithm
// WrapperOnDiskSimpleEd
// WrapperOnDiskSimpleEdNorm
// WrapperOnDiskSimpleJacc
// WrapperOnDiskSimpleCos
// WrapperOnDiskSimpleDice

// wrappers using adaptive merging algorithm
// WrapperOnDiskAdaptEd
// WrapperOnDiskAdaptEdNorm
// WrapperOnDiskAdaptJacc
// WrapperOnDiskAdaptCos
// WrapperOnDiskAdaptDice

// see typedefs in wrapperondisk.h
// ALSO POSSIBLE TO SPECIFY SIMMETRIC AS TEMPLATE PARAMETER: WrapperSimple<SimilarityMetric>
void wrapperOnDiskSimpleExample() {
  cout << "----- EXAMPLE: WrapperOnDisk -----" << endl;

  GramGenFixedLen gramGen(2);  
  StringContainerRM strContainer(true);
  strContainer.initStatsCollector(&gramGen);
  strContainer.createAndOpen("wrapperContainer.rm");
  //strContainer.openContainer("wrapperContainer.rm");
  strContainer.fillContainer(strPairColl.begin(), strPairColl.end());


  uint id;
  string tmp;
  strContainer.retrieveString(tmp, id,0);
  cout<<tmp.c_str()<<" "<<id<<endl;
  strContainer.retrieveString(tmp, id,1983153);
  cout<<tmp.c_str()<<" "<<id<<endl;
  
  // create wrapper and build index
  // params: stringcontainer, use partitioning filter?, gramlength, use pre- and postfixing or string?
  WrapperOnDiskSimpleEd wrapper(&strContainer, "invLists.ix", 2, true, true);
  wrapper.buildIndex();

  // perform search

  float editDistance = 1.0f;
  string queryString = "Agua?Sal?Wash";
  vector<unsigned> resultStringIDs;

 
  wrapper.search(queryString, editDistance, resultStringIDs);
  cout<<"resultStringIDs.size:"<<resultStringIDs.size()<<endl;
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string strtmp;
    uint tmpid;
    strContainer.retrieveString(strtmp, tmpid, resultStringIDs[i]);
    cout << strtmp <<" "<<tmpid<< endl;
  }

 

  // save index
  wrapper.saveIndex("wrapperIndex.ix");

  // load index
  wrapper.loadIndex("wrapperIndex.ix");

  // perform search on loaded index
  resultStringIDs.clear();
  wrapper.search(queryString, editDistance, resultStringIDs);

  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    uint tmpid;
    strContainer.retrieveString(tmp, tmpid, resultStringIDs[i]);
    cout << tmp <<" "<<tmpid<< endl;
  }

  cout << "-----------------------------------------" << endl << endl;  
}
*/

void customerizeMerger() {
  cout << "----- ondisk ADVANCED USAGE ----" << endl;

  // create gramgenerator and similarity metric
  GramGenFixedLen gramGen(2,true); //use prePostfix
  SimMetricEd simMetric(gramGen);
  
  // using disk-based string container
  StringContainerRM strContainer(true);
  //strContainer.createAndOpen("collection.rm");
  strContainer.initStatsCollector(&gramGen);
  strContainer.openContainer("collection.rm");
  //strContainer.fillContainer(strPairColl.begin(), strPairColl.end()); // fill the container from dictionary
  // params: container, gramgen, disableStreamBuffer, index filename, bytes per run, max str len, fanout
  FtIndexerOnDisk<> indexer(&strContainer, &gramGen, false, "invlists.ix");
  indexer.autoAddPartFilter();
  //indexer.buildIndex();
  
  // scanCountMerger
  //ScanCountMerger<vector<unsigned> > merger(strPairColl.size());

  // create a mergeSkipMerger
  //MergeSkipMerger<vector<unsigned> > merger;

   //default merger
   //MergeOptMerger<> merger;
   OnDiskMergerSimple<>  merger(strContainer.size()) ;

  // create searcher passing merger and indexer
  // specify all template arguments, need to specify merger if not default (MergeOptMerger)
  FtSearcherOnDisk<FtIndexerOnDisk<>, OnDiskMergerSimple<> > searcher(&merger,&indexer);

  
  vector<unsigned> resultStringIDs;
  Query query("Agua?Sal?Wash", simMetric, 1.0f);
 

  //indexer.saveIndex("exampleindex.ix");
  //cout << "LOADING INDEX" << endl;
  indexer.loadIndex("exampleindex.ix");
  
  resultStringIDs.clear();
  //searcher.setFtIndexer(&indexer);
  searcher.search(query, resultStringIDs);  
  cout << "SIMILAR STRINGS: " << endl;
  for(unsigned i = 0; i < resultStringIDs.size(); i++) {
    string tmp;
    strContainer.retrieveString(tmp, resultStringIDs[i]);
    cout << tmp << endl;
  }
  
  cout << "-------------------------" << endl << endl;
}


